@@ -1,29 +1,17 @@
-/* This file is part of the KDE libraries
+/*
+    This file is part of the KDE libraries
 
-   Copyright (c) 2000,2001 Dawit Alemayehu <adawit@kde.org>
-   Copyright (c) 2000,2001 Carsten Pfeiffer <pfeiffer@kde.org>
-   Copyright (c) 2000 Stefan Schimanski <1Stein@gmx.de>
+    SPDX-FileCopyrightText: 2000, 2001 Dawit Alemayehu <adawit@kde.org>
+    SPDX-FileCopyrightText: 2000, 2001 Carsten Pfeiffer <pfeiffer@kde.org>
+    SPDX-FileCopyrightText: 2000 Stefan Schimanski <1Stein@gmx.de>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License (LGPL) as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.1-or-later
 */
 
 #include "khistorycombobox.h"
 
 #include <kpixmapprovider.h>
-#include <kstandardshortcut.h>
+#include <KStandardShortcut>
 
 #include <QAbstractItemView>
 #include <QApplication>
@@ -62,7 +50,9 @@ public:
      */
     QString typedText;
 
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 66)
     KPixmapProvider *pixmapProvider = nullptr;
+#endif
     KHistoryComboBox * const q_ptr;
 
     /**
@@ -76,6 +66,8 @@ public:
      */
     bool rotated = false;
 
+    std::function<QIcon(QString)> iconProvider;
+
     Q_DECLARE_PUBLIC(KHistoryComboBox)
 };
 
@@ -92,7 +84,9 @@ void KHistoryComboBoxPrivate::init(bool useCompletion)
     q->setInsertPolicy(KHistoryComboBox::NoInsert);
     currentIndex = -1;
     rotated = false;
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 66)
     pixmapProvider = nullptr;
+#endif
 
     // obey HISTCONTROL setting
     QByteArray histControl = qgetenv("HISTCONTROL");
@@ -130,8 +124,10 @@ KHistoryComboBox::KHistoryComboBox(bool useCompletion,
 
 KHistoryComboBox::~KHistoryComboBox()
 {
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 66)
     Q_D(KHistoryComboBox);
     delete d->pixmapProvider;
+#endif
 }
 
 void KHistoryComboBox::setHistoryItems(const QStringList &items)
@@ -203,7 +199,7 @@ void KHistoryComboBoxPrivate::_k_addContextMenuItems(QMenu *menu)
     if (menu) {
         menu->addSeparator();
         QAction *clearHistory = menu->addAction(QIcon::fromTheme(QStringLiteral("edit-clear-history")),
-                                                KHistoryComboBox::tr("Clear &History"),
+                                                KHistoryComboBox::tr("Clear &History", "@action:inmenu"),
                                                 q, SLOT(_k_clear()));
         if (!q->count()) {
             clearHistory->setEnabled(false);
@@ -237,8 +233,12 @@ void KHistoryComboBox::addToHistory(const QString &item)
     }
 
     // now add the item
-    if (d->pixmapProvider) {
+    if (d->iconProvider) {
+        insertItem(0, d->iconProvider(item), item);
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 66)
+    } else if (d->pixmapProvider) {
         insertItem(0, d->pixmapProvider->pixmapFor(item, iconSize().height()), item);
+#endif
     } else {
         insertItem(0, item);
     }
@@ -400,7 +400,7 @@ void KHistoryComboBox::wheelEvent(QWheelEvent *ev)
         return;
     }
     // Otherwise make it change the text without emitting activated
-    if (ev->delta() > 0) {
+    if (ev->angleDelta().y() > 0) {
         d->rotateUp();
     } else {
         d->rotateDown();
@@ -408,6 +408,7 @@ void KHistoryComboBox::wheelEvent(QWheelEvent *ev)
     ev->accept();
 }
 
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 66)
 void KHistoryComboBox::setPixmapProvider(KPixmapProvider *provider)
 {
     Q_D(KHistoryComboBox);
@@ -427,24 +428,32 @@ void KHistoryComboBox::setPixmapProvider(KPixmapProvider *provider)
         insertItems(items);
     }
 }
+#endif
+
+void KHistoryComboBox::setIconProvider(std::function<QIcon(const QString &)> providerFunction)
+{
+    Q_D(KHistoryComboBox);
+    d->iconProvider = providerFunction;
+}
 
 void KHistoryComboBox::insertItems(const QStringList &items)
 {
     Q_D(KHistoryComboBox);
-    QStringList::ConstIterator it = items.constBegin();
-    const QStringList::ConstIterator itEnd = items.constEnd();
 
-    while (it != itEnd) {
-        const QString item = *it;
-        if (!item.isEmpty()) {   // only insert non-empty items
-            if (d->pixmapProvider)
-                addItem(d->pixmapProvider->pixmapFor(item, iconSize().height()),
-                        item);
-            else {
-                addItem(item);
-            }
+    for (const QString item : items) {
+        if (item.isEmpty()) {
+            continue;
+        }
+
+        if (d->iconProvider) {
+            addItem(d->iconProvider(item), item);
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 66)
+        } else if (d->pixmapProvider) {
+            addItem(d->pixmapProvider->pixmapFor(item, iconSize().height()), item);
+#endif
+        } else {
+            addItem(item);
         }
-        ++it;
     }
 }
 
@@ -463,7 +472,16 @@ void KHistoryComboBoxPrivate::_k_simulateActivated(const QString &text)
        which is perhaps reasonable. Generate the signal ourselves if that's the case.
     */
     if ((q->insertPolicy() == q->NoInsert && q->findText(text, Qt::MatchFixedString | Qt::MatchCaseSensitive) == -1)) {
+#if QT_DEPRECATED_SINCE(5, 15) || QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
         emit q->activated(text);
+QT_WARNING_POP
+#endif
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+        emit q->textActivated(text);
+#endif
     }
 
     /*
@@ -471,15 +489,26 @@ void KHistoryComboBoxPrivate::_k_simulateActivated(const QString &text)
        InsertAtCurrent
     */
     else if (q->insertPolicy() != q->InsertAtCurrent && q->count() >= q->maxCount()) {
+#if QT_DEPRECATED_SINCE(5, 15) || QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
         emit q->activated(text);
+QT_WARNING_POP
+#endif
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+        emit q->textActivated(text);
+#endif
     }
 }
 
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 66)
 KPixmapProvider *KHistoryComboBox::pixmapProvider() const
 {
     Q_D(const KHistoryComboBox);
     return d->pixmapProvider;
 }
+#endif
 
 void KHistoryComboBox::reset()
 {
